@@ -7,24 +7,18 @@ use crate::{
     Bus, Error, Message, Relay,
 };
 use core::{
-    any::TypeId,
+    any::{Any, TypeId},
     fmt,
     marker::PhantomData,
     mem,
     pin::Pin,
+    sync::atomic::{AtomicBool, AtomicI64, Ordering},
     task::{Context, Poll},
 };
-use futures::Future;
-use futures::{future::poll_fn, FutureExt};
-use std::{
-    any::Any,
-    borrow::Cow,
-    sync::{
-        atomic::{AtomicBool, AtomicU64, Ordering},
-        Arc,
-    },
-};
+use futures::{future::poll_fn, Future, FutureExt};
+use std::{borrow::Cow, sync::Arc};
 use tokio::sync::{oneshot, Notify};
+
 struct SlabCfg;
 impl sharded_slab::Config for SlabCfg {
     const RESERVED_BITS: usize = 1;
@@ -38,6 +32,7 @@ pub trait SendUntypedReceiver: Send + Sync {
         &self,
         _mid: u64,
         _msg: Box<dyn Message>,
+        _req: bool,
         _bus: &Bus,
     ) -> Result<(), SendError<Box<dyn Message>>> {
         unimplemented!()
@@ -45,7 +40,7 @@ pub trait SendUntypedReceiver: Send + Sync {
 }
 
 pub trait SendTypedReceiver<M: Message>: Sync {
-    fn send(&self, mid: u64, msg: M, bus: &Bus) -> Result<(), SendError<M>>;
+    fn send(&self, mid: u64, msg: M, req: bool, bus: &Bus) -> Result<(), SendError<M>>;
 }
 
 pub trait ReciveTypedReceiver<M, E>: Sync
@@ -56,7 +51,7 @@ where
     fn poll_events(&self, ctx: &mut Context<'_>, bus: &Bus) -> Poll<Event<M, E>>;
 }
 
-pub trait ReciveUnypedReceiver: Sync {
+pub trait ReciveUntypedReceiver: Sync {
     fn poll_events(
         &self,
         ctx: &mut Context<'_>,
@@ -103,6 +98,7 @@ pub trait ReceiverTrait: TypeTagAccept + Send + Sync {
         &self,
         mid: u64,
         msg: Box<dyn Message>,
+        req: bool,
         bus: &Bus,
     ) -> Result<(), Error<Box<dyn Message>>>;
     fn add_response_listener(
@@ -125,6 +121,7 @@ pub trait ReceiverTrait: TypeTagAccept + Send + Sync {
 
     fn try_reserve(&self, tt: &TypeTag) -> Option<Permit>;
     fn reserve_notify(&self, tt: &TypeTag) -> Arc<Notify>;
+    fn increment_processing(&self, tt: &TypeTag);
 
     fn start_polling(
         self: Arc<Self>,
@@ -154,6 +151,7 @@ pub enum Action {
 pub enum Event<M, E: StdSyncSendError> {
     Response(u64, Result<M, Error<(), E>>),
     Synchronized(Result<(), Error<(), E>>),
+    Finished(u64),
     Error(E),
     InitFailed(Error<(), E>),
     Stats(Stats),
@@ -224,6 +222,16 @@ where
                             }
                         }
 
+                        Event::Finished(n) => {
+                            self.context.processing.fetch_sub(n as _, Ordering::SeqCst);
+
+                            if n > 1 {
+                                self.context.response.notify_waiters();
+                            } else {
+                                self.context.response.notify_one();
+                            }
+                        }
+
                         _ => unimplemented!(),
                     }
                 }
@@ -252,9 +260,9 @@ where
                     .send(resp.map_err(|e| e.into_dyn()).map(|x| x.into_boxed()))
                     .unwrap(),
 
-                Waiter::BoxedWithError(sender) => sender
-                    .send(resp.map(|x| x.into_boxed()))
-                    .unwrap(),
+                Waiter::BoxedWithError(sender) => {
+                    sender.send(resp.map(|x| x.into_boxed())).unwrap()
+                }
             }
         }
 
@@ -349,6 +357,7 @@ where
         &self,
         mid: u64,
         boxed_msg: Box<dyn Message>,
+        req: bool,
         bus: &Bus,
     ) -> Result<(), Error<Box<dyn Message>>> {
         let boxed = boxed_msg
@@ -356,7 +365,7 @@ where
             .downcast::<M>()
             .map_err(|_| Error::MessageCastError)?;
 
-        Ok(SendTypedReceiver::send(&self.inner, mid, *boxed, bus)
+        Ok(SendTypedReceiver::send(&self.inner, mid, *boxed, req, bus)
             .map_err(|err| Error::from(err.into_boxed()))?)
     }
 
@@ -365,11 +374,11 @@ where
             msg_type_tag: M::type_tag_(),
             resp_type_tag: Some(R::type_tag_()),
             err_type_tag: Some(E::type_tag_()),
-            
+
             has_queue: true,
             queue_capacity: self.context.limit as _,
             queue_size: self.context.processing.load(Ordering::Relaxed) as _,
-            
+
             ..Default::default()
         }
     }
@@ -424,7 +433,7 @@ where
         loop {
             let count = self.context.processing.load(Ordering::Relaxed);
 
-            if count < self.context.limit {
+            if count < self.context.limit as _ {
                 let res = self.context.processing.compare_exchange(
                     count,
                     count + 1,
@@ -454,6 +463,10 @@ where
     ) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>> {
         self.start_polling_events()
     }
+
+    fn increment_processing(&self, _tt: &TypeTag) {
+        self.context.processing.fetch_add(1, Ordering::SeqCst);
+    }
 }
 
 pub struct Permit {
@@ -526,7 +539,10 @@ impl<'a> AnyWrapperRef<'a> {
     where
         R: Message,
         E: StdSyncSendError,
-        S: WrapperReturnTypeOnly<R> + WrapperErrorTypeOnly<E> + WrapperReturnTypeAndError<R, E> + 'static,
+        S: WrapperReturnTypeOnly<R>
+            + WrapperErrorTypeOnly<E>
+            + WrapperReturnTypeAndError<R, E>
+            + 'static,
     {
         let wrapper_r = rcvr as &(dyn WrapperReturnTypeOnly<R>);
         let wrapper_e = rcvr as &(dyn WrapperErrorTypeOnly<E>);
@@ -569,7 +585,9 @@ impl<'a> AnyWrapperRef<'a> {
     }
 
     #[inline]
-    pub fn cast_error_only<E: StdSyncSendError>(&'a self) -> Option<&'a dyn WrapperErrorTypeOnly<E>> {
+    pub fn cast_error_only<E: StdSyncSendError>(
+        &'a self,
+    ) -> Option<&'a dyn WrapperErrorTypeOnly<E>> {
         if self.wrapper_e.0 != TypeId::of::<dyn WrapperErrorTypeOnly<E>>() {
             return None;
         }
@@ -626,7 +644,7 @@ impl fmt::Display for ReceiverStats {
 
 struct ReceiverContext {
     limit: u64,
-    processing: AtomicU64,
+    processing: AtomicI64,
     need_flush: AtomicBool,
     ready_flag: AtomicBool,
     flushed: Notify,
@@ -685,7 +703,7 @@ impl Receiver {
                 waiters: sharded_slab::Slab::new_with_config::<SlabCfg>(),
                 context: Arc::new(ReceiverContext {
                     limit,
-                    processing: AtomicU64::new(0),
+                    processing: AtomicI64::new(0),
                     need_flush: AtomicBool::new(false),
                     ready_flag: AtomicBool::new(false),
                     init_sent: AtomicBool::new(false),
@@ -752,17 +770,18 @@ impl Receiver {
         bus: &Bus,
         mid: u64,
         msg: M,
+        req: bool,
         mut permit: Permit,
     ) -> Result<(), Error<M>> {
         let res = if let Some(any_receiver) = self.inner.typed() {
             any_receiver
                 .cast_send_typed::<M>()
                 .unwrap()
-                .send(mid, msg, bus)
+                .send(mid, msg, req, bus)
                 .map_err(Into::into)
         } else {
             self.inner
-                .send_boxed(mid, msg.into_boxed(), bus)
+                .send_boxed(mid, msg.into_boxed(), req, bus)
                 .map_err(|err| err.map_msg(|b| *b.as_any_boxed().downcast::<M>().unwrap()))
                 .map(|_| ())
         };
@@ -779,16 +798,19 @@ impl Receiver {
         bus: &Bus,
         mid: u64,
         msg: M,
+        req: bool,
     ) -> Result<(), Error<M>> {
+        self.inner.increment_processing(&M::type_tag_());
+
         let res = if let Some(any_receiver) = self.inner.typed() {
             any_receiver
                 .cast_send_typed::<M>()
                 .unwrap()
-                .send(mid, msg, bus)
+                .send(mid, msg, req, bus)
                 .map_err(Into::into)
         } else {
             self.inner
-                .send_boxed(mid, msg.into_boxed(), bus)
+                .send_boxed(mid, msg.into_boxed(), req, bus)
                 .map_err(|err| err.map_msg(|b| *b.as_any_boxed().downcast::<M>().unwrap()))
                 .map(|_| ())
         };
@@ -803,11 +825,12 @@ impl Receiver {
         bus: &Bus,
         mid: u64,
         msg: Box<dyn Message>,
+        req: bool,
         mut permit: Permit,
     ) -> Result<(), Error<Box<dyn Message>>> {
-        let res = self.inner.send_boxed(mid, msg, bus);
-        self.inner.set_need_flush();
+        let res = self.inner.send_boxed(mid, msg, req, bus);
         permit.fuse = true;
+        self.inner.set_need_flush();
         res
     }
 
@@ -836,7 +859,13 @@ impl Receiver {
     #[inline]
     pub(crate) fn add_response_waiter_boxed_we<E: StdSyncSendError>(
         &self,
-    ) -> Result<(u64, impl Future<Output = Result<Box<dyn Message>, Error<(), E>>>), Error> {
+    ) -> Result<
+        (
+            u64,
+            impl Future<Output = Result<Box<dyn Message>, Error<(), E>>>,
+        ),
+        Error,
+    > {
         if let Some(any_wrapper) = self.inner.wrapper() {
             let (tx, rx) = oneshot::channel();
             let mid = any_wrapper
