@@ -12,21 +12,27 @@ use crate::{
     builder::ReceiverSubscriberBuilder,
     error::{Error, SendError, StdSyncSendError},
     receiver::{Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver},
-    receivers::{fix_type, Request},
+    receivers::Request,
     AsyncHandler, Bus, Message, Untyped,
 };
 
 use super::{BufferUnorderedConfig, BufferUnorderedStats};
 
-use futures::{stream::FuturesUnordered, Future, StreamExt};
+use futures::Future;
 use parking_lot::Mutex;
-use tokio::sync::mpsc;
+use tokio::sync::mpsc::{self, UnboundedSender};
 
 buffer_unordered_poller_macro!(
     T,
     AsyncHandler,
-    |mid, msg, bus, ut: Arc<T>, _stats: Arc<BufferUnorderedStats>| {
-        async move { (mid, ut.handle(msg, &bus).await) }
+    |mid, msg, bus, ut: Arc<T>, stx: UnboundedSender<_>, task_permit| {
+        tokio::spawn(async move {
+            let resp = ut.handle(msg, &bus).await;
+            drop(task_permit);
+
+            stx.send(Event::Response(mid, resp.map_err(Error::Other)))
+                .unwrap();
+        })
     },
     |bus, ut: Arc<T>| { async move { ut.sync(&bus).await } }
 );
@@ -115,14 +121,14 @@ where
     R: Message,
     E: StdSyncSendError,
 {
-    fn send(&self, mid: u64, m: M, _bus: &Bus) -> Result<(), SendError<M>> {
-        match self.tx.send(Request::Request(mid, m)) {
+    fn send(&self, mid: u64, m: M, req: bool, _bus: &Bus) -> Result<(), SendError<M>> {
+        match self.tx.send(Request::Request(mid, m, req)) {
             Ok(_) => {
                 self.stats.buffer.fetch_add(1, Ordering::Relaxed);
 
                 Ok(())
             }
-            Err(mpsc::error::SendError(Request::Request(_, msg))) => Err(SendError::Closed(msg)),
+            Err(mpsc::error::SendError(Request::Request(_, msg, _))) => Err(SendError::Closed(msg)),
             _ => unimplemented!(),
         }
     }
