@@ -12,25 +12,25 @@ use crate::{
     builder::ReceiverSubscriberBuilder,
     error::{Error, SendError, StdSyncSendError},
     receiver::{Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver},
-    receivers::{fix_type, Request},
+    receivers::Request,
     AsyncBatchHandler, Bus, Message, Untyped,
 };
 
 use super::{BufferUnorderedBatchedConfig, BufferUnorderedBatchedStats};
-use futures::{stream::FuturesUnordered, Future, StreamExt};
+use futures::Future;
 use parking_lot::Mutex;
-use tokio::sync::mpsc;
+use tokio::sync::mpsc::{self, UnboundedSender};
 
 buffer_unordered_batch_poller_macro!(
     T,
     AsyncBatchHandler,
-    |buffer_clone, bus, ut: Arc<T>, stats: Arc<BufferUnorderedBatchedStats>, buffer_mid_clone| {
-        async move {
-            let resp = ut.handle(buffer_clone, &bus).await;
-            stats.parallel.fetch_sub(1, Ordering::Relaxed);
+    |mids: Vec<_>, msgs, bus, ut: Arc<T>, task_permit, stx: UnboundedSender<_>| {
+        tokio::spawn(async move {
+            let resp = ut.handle(msgs, &bus).await;
+            drop(task_permit);
 
-            (buffer_mid_clone, resp)
-        }
+            crate::process_batch_result!(resp, mids, stx);
+        })
     },
     |bus, ut: Arc<T>| { async move { ut.sync(&bus).await } }
 );
@@ -122,14 +122,14 @@ where
     R: Message,
     E: StdSyncSendError,
 {
-    fn send(&self, mid: u64, m: M, _bus: &Bus) -> Result<(), SendError<M>> {
-        match self.tx.send(Request::Request(mid, m)) {
+    fn send(&self, mid: u64, m: M, req: bool, _bus: &Bus) -> Result<(), SendError<M>> {
+        match self.tx.send(Request::Request(mid, m, req)) {
             Ok(_) => {
                 self.stats.buffer.fetch_add(1, Ordering::Relaxed);
 
                 Ok(())
             }
-            Err(mpsc::error::SendError(Request::Request(_, msg))) => Err(SendError::Closed(msg)),
+            Err(mpsc::error::SendError(Request::Request(_, msg, _))) => Err(SendError::Closed(msg)),
             _ => unimplemented!(),
         }
     }
