@@ -13,27 +13,24 @@ use crate::{
     builder::ReceiverSubscriberBuilder,
     error::{Error, SendError, StdSyncSendError},
     receiver::{Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver},
-    receivers::{fix_type, Request},
+    receivers::Request,
     BatchHandler, Bus, Message, Untyped,
 };
 
-use futures::{stream::FuturesUnordered, Future, StreamExt};
+use futures::Future;
 use parking_lot::Mutex;
-use tokio::sync::mpsc;
+use tokio::sync::mpsc::{self, UnboundedSender};
 
 buffer_unordered_batch_poller_macro!(
     T,
     BatchHandler,
-    |buffer_clone, bus, ut: Arc<T>, stats: Arc<BufferUnorderedBatchedStats>, buffer_mid_clone| {
-        async move {
-            let resp = tokio::task::spawn_blocking(move || {
-                (buffer_mid_clone, ut.handle(buffer_clone, &bus))
-            })
-            .await
-            .unwrap();
-            stats.parallel.fetch_sub(1, Ordering::Relaxed);
-            resp
-        }
+    |mids: Vec<_>, msgs, bus, ut: Arc<T>, task_permit, stx: UnboundedSender<_>| {
+        tokio::task::spawn_blocking(move || {
+            let resp = ut.handle(msgs, &bus);
+            drop(task_permit);
+
+            crate::process_batch_result!(resp, mids, stx);
+        })
     },
     |bus, ut: Arc<T>| {
         async move {
@@ -131,14 +128,14 @@ where
     R: Message,
     E: StdSyncSendError,
 {
-    fn send(&self, mid: u64, m: M, _bus: &Bus) -> Result<(), SendError<M>> {
-        match self.tx.send(Request::Request(mid, m)) {
+    fn send(&self, mid: u64, m: M, req: bool, _bus: &Bus) -> Result<(), SendError<M>> {
+        match self.tx.send(Request::Request(mid, m, req)) {
             Ok(_) => {
                 self.stats.buffer.fetch_add(1, Ordering::Relaxed);
 
                 Ok(())
             }
-            Err(mpsc::error::SendError(Request::Request(_, msg))) => Err(SendError::Closed(msg)),
+            Err(mpsc::error::SendError(Request::Request(_, msg, _))) => Err(SendError::Closed(msg)),
             _ => unimplemented!(),
         }
     }
