@@ -9,7 +9,7 @@ use crate::{
     builder::ReceiverSubscriberBuilder,
     error::{Error, SendError, StdSyncSendError},
     receiver::{Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver},
-    receivers::{fix_into_iter, fix_type, Request},
+    receivers::Request,
     AsyncBatchSynchronizedHandler, Bus, Message, Untyped,
 };
 
@@ -17,15 +17,20 @@ use futures::Future;
 
 use super::SynchronizedBatchedConfig;
 
-use tokio::sync::{mpsc, Mutex};
+use tokio::sync::{
+    mpsc::{self, UnboundedSender},
+    Mutex,
+};
 
 batch_synchronized_poller_macro! {
     T,
     AsyncBatchSynchronizedHandler,
-    |mids, buffer, bus, ut: Arc<Mutex<T>>| {
-        async move {
-            (mids, ut.lock().await.handle(buffer, &bus).await)
-        }
+    |mids: Vec<_>, msgs, bus, ut: Arc<Mutex<T>>, stx: UnboundedSender<_>| {
+        tokio::spawn(async move {
+            let resp = ut.lock().await.handle(msgs, &bus).await;
+
+            crate::process_batch_result!(resp, mids, stx);
+        })
     },
     |bus, ut: Arc<Mutex<T>>| { async move { ut.lock().await.sync(&bus).await } }
 }
@@ -99,10 +104,10 @@ where
     R: Message,
     E: StdSyncSendError,
 {
-    fn send(&self, mid: u64, m: M, _bus: &Bus) -> Result<(), SendError<M>> {
-        match self.tx.send(Request::Request(mid, m)) {
+    fn send(&self, mid: u64, m: M, req: bool, _bus: &Bus) -> Result<(), SendError<M>> {
+        match self.tx.send(Request::Request(mid, m, req)) {
             Ok(_) => Ok(()),
-            Err(mpsc::error::SendError(Request::Request(_, msg))) => Err(SendError::Closed(msg)),
+            Err(mpsc::error::SendError(Request::Request(_, msg, _))) => Err(SendError::Closed(msg)),
             _ => unimplemented!(),
         }
     }
