@@ -9,28 +9,31 @@ use crate::{
     builder::ReceiverSubscriberBuilder,
     error::{Error, SendError, StdSyncSendError},
     receiver::{Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver},
-    receivers::{fix_into_iter, fix_type, Request},
+    receivers::Request,
     BatchSynchronizedHandler, Bus, Message, Untyped,
 };
 
 use super::SynchronizedBatchedConfig;
 use futures::{executor::block_on, Future};
-use tokio::sync::{mpsc, Mutex};
+use tokio::sync::{
+    mpsc::{self, UnboundedSender},
+    Mutex,
+};
 
 batch_synchronized_poller_macro! {
     T,
     BatchSynchronizedHandler,
-    |mids, buffer, bus, ut: Arc<Mutex<T>>| async move  {
-        (mids, tokio::task::spawn_blocking(move || {
-            block_on(ut.lock()).handle(buffer, &bus)
+    |mids: Vec<_>, msgs, bus, ut: Arc<Mutex<T>>, stx: UnboundedSender<_>| {
+        tokio::task::spawn_blocking(move || {
+            let resp = block_on(ut.lock()).handle(msgs, &bus);
+
+            crate::process_batch_result!(resp, mids, stx);
         })
-        .await
-        .unwrap())
     },
     |bus, ut: Arc<Mutex<T>>| async move {
         tokio::task::spawn_blocking(move || block_on(ut.lock()).sync(&bus))
-                        .await
-                        .unwrap()
+            .await
+            .unwrap()
     }
 }
 
@@ -103,10 +106,10 @@ where
     R: Message,
     E: StdSyncSendError,
 {
-    fn send(&self, mid: u64, m: M, _bus: &Bus) -> Result<(), SendError<M>> {
-        match self.tx.send(Request::Request(mid, m)) {
+    fn send(&self, mid: u64, m: M, req: bool, _bus: &Bus) -> Result<(), SendError<M>> {
+        match self.tx.send(Request::Request(mid, m, req)) {
             Ok(_) => Ok(()),
-            Err(mpsc::error::SendError(Request::Request(_, msg))) => Err(SendError::Closed(msg)),
+            Err(mpsc::error::SendError(Request::Request(_, msg, _))) => Err(SendError::Closed(msg)),
             _ => unimplemented!(),
         }
     }
