@@ -12,17 +12,26 @@ use crate::{
     builder::ReceiverSubscriberBuilder,
     error::{Error, SendError, StdSyncSendError},
     receiver::{Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver},
-    receivers::{fix_type, Request},
+    receivers::Request,
     AsyncSynchronizedHandler, Bus, Message, Untyped,
 };
-use tokio::sync::{mpsc, Mutex};
+use tokio::sync::{
+    mpsc::{self, UnboundedSender},
+    Mutex,
+};
 
 synchronized_poller_macro! {
     T,
     AsyncSynchronizedHandler,
-    |mid, msg, bus, ut: Arc<Mutex<T>>| async move {
-        (mid, ut.lock().await.handle(msg, &bus).await)
+    |mid, msg, bus, ut: Arc<Mutex<T>>, stx: UnboundedSender<_>| {
+        tokio::spawn(async move {
+            let resp = ut.lock().await.handle(msg, &bus).await;
+
+            stx.send(Event::Response(mid, resp.map_err(Error::Other)))
+                .unwrap();
+        })
     },
+
     |bus, ut: Arc<Mutex<T>>| async move {
         ut.lock().await.sync(&bus).await
     }
@@ -96,10 +105,10 @@ where
     R: Message,
     E: StdSyncSendError,
 {
-    fn send(&self, mid: u64, m: M, _bus: &Bus) -> Result<(), SendError<M>> {
-        match self.tx.send(Request::Request(mid, m)) {
+    fn send(&self, mid: u64, m: M, req: bool, _bus: &Bus) -> Result<(), SendError<M>> {
+        match self.tx.send(Request::Request(mid, m, req)) {
             Ok(_) => Ok(()),
-            Err(mpsc::error::SendError(Request::Request(_, msg))) => Err(SendError::Closed(msg)),
+            Err(mpsc::error::SendError(Request::Request(_, msg, _))) => Err(SendError::Closed(msg)),
             _ => unimplemented!(),
         }
     }
