@@ -12,25 +12,29 @@ use crate::{
     builder::ReceiverSubscriberBuilder,
     error::{Error, SendError, StdSyncSendError},
     receiver::{Action, Event, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver},
-    receivers::{fix_type, Request},
+    receivers::Request,
     Bus, Message, SynchronizedHandler, Untyped,
 };
-use tokio::sync::{mpsc, Mutex};
+use tokio::sync::{
+    mpsc::{self, UnboundedSender},
+    Mutex,
+};
 
 synchronized_poller_macro! {
     T,
     SynchronizedHandler,
-    |mid, msg, bus, ut: Arc<Mutex<T>>| async move {
-        (mid, tokio::task::spawn_blocking(move || {
-            block_on(ut.lock()).handle(msg, &bus)
+    |mid, msg, bus, ut: Arc<Mutex<T>>, stx: UnboundedSender<_>| {
+        tokio::task::spawn_blocking(move || {
+            let resp = block_on(ut.lock()).handle(msg, &bus);
+
+            stx.send(Event::Response(mid, resp.map_err(Error::Other)))
+                .unwrap();
         })
-        .await
-        .unwrap())
     },
     |bus, ut: Arc<Mutex<T>>| async move {
         tokio::task::spawn_blocking(move || block_on(ut.lock()).sync(&bus))
-                        .await
-                        .unwrap()
+            .await
+            .unwrap()
     }
 }
 
@@ -102,10 +106,10 @@ where
     R: Message,
     E: StdSyncSendError,
 {
-    fn send(&self, mid: u64, m: M, _bus: &Bus) -> Result<(), SendError<M>> {
-        match self.tx.send(Request::Request(mid, m)) {
+    fn send(&self, mid: u64, m: M, req: bool, _bus: &Bus) -> Result<(), SendError<M>> {
+        match self.tx.send(Request::Request(mid, m, req)) {
             Ok(_) => Ok(()),
-            Err(mpsc::error::SendError(Request::Request(_, msg))) => Err(SendError::Closed(msg)),
+            Err(mpsc::error::SendError(Request::Request(_, msg, _))) => Err(SendError::Closed(msg)),
             _ => unimplemented!(),
         }
     }
