@@ -1,20 +1,23 @@
-use crate::{Bus, Event, Message, Permit, ReciveUnypedReceiver, TypeTag, error::Error, receiver::{
+use crate::{
+    error::Error,
+    receiver::{
         Action, AnyReceiver, AnyWrapperRef, PermitDrop, ReceiverTrait, SendUntypedReceiver,
         TypeTagAccept,
-    }, stats::Stats};
+    },
+    stats::Stats,
+    Bus, Event, Message, Permit, ReciveUntypedReceiver, TypeTag,
+};
+use core::{
+    pin::Pin,
+    sync::atomic::{AtomicBool, AtomicU64, Ordering},
+};
 use dashmap::DashMap;
 use futures::{future::poll_fn, Future};
-use std::{
-    pin::Pin,
-    sync::{
-        atomic::{AtomicBool, AtomicU64, Ordering},
-        Arc,
-    },
-};
+use std::sync::Arc;
 use tokio::sync::{oneshot, Notify};
 
-pub trait Relay: TypeTagAccept + SendUntypedReceiver + ReciveUnypedReceiver + 'static {}
-impl<T: TypeTagAccept + SendUntypedReceiver + ReciveUnypedReceiver + 'static> Relay for T {}
+pub trait Relay: TypeTagAccept + SendUntypedReceiver + ReciveUntypedReceiver + 'static {}
+impl<T: TypeTagAccept + SendUntypedReceiver + ReciveUntypedReceiver + 'static> Relay for T {}
 
 struct SlabCfg;
 impl sharded_slab::Config for SlabCfg {
@@ -115,9 +118,10 @@ where
         &self,
         mid: u64,
         boxed_msg: Box<dyn Message>,
+        req: bool,
         bus: &Bus,
     ) -> Result<(), Error<Box<dyn Message>>> {
-        Ok(self.inner.send_msg(mid, boxed_msg, bus)?)
+        Ok(self.inner.send_msg(mid, boxed_msg, req, bus)?)
     }
 
     fn need_flush(&self) -> bool {
@@ -212,6 +216,13 @@ where
         self.context.init_sent.load(Ordering::SeqCst)
     }
 
+    fn increment_processing(&self, tt: &TypeTag) {
+        self.context
+            .receivers
+            .get(tt)
+            .map(|r| r.processing.fetch_add(1, Ordering::SeqCst));
+    }
+
     fn start_polling(
         self: Arc<Self>,
     ) -> Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>> {
@@ -240,7 +251,7 @@ where
                             break;
                         }
                         Event::Flushed => {
-                            self.context.need_flush.store(true, Ordering::SeqCst);
+                            self.context.need_flush.store(false, Ordering::SeqCst);
                             self.context.flushed.notify_waiters()
                         }
                         Event::Synchronized(_res) => self.context.synchronized.notify_waiters(),
